defmodule LinkShortenerWeb.LinkLive.FormComponent do
  use LinkShortenerWeb, :live_component

  alias LinkShortener.Links

  @impl true
  def render(assigns) do
    ~H"""
    <div>
      <.header>
        <%= @title %>
        <:subtitle>Use this form to manage link records.</:subtitle>
      </.header>

      <.simple_form
        for={@form}
        id="link-form"
        phx-target={@myself}
        phx-change="validate"
        phx-submit="save"
      >
        <.input field={@form[:name]} type="text" label="Name" placeholder="Enter a name here" />
        <.input field={@form[:url]} type="text" label="Url" placeholder="Enter an url here" />
        <.input field={@form[:shorten]} type="text" label="Shorten" placeholder="Enter a shorten here" />
        <:actions>
          <.button phx-disable-with="Saving...">Save Link</.button>
        </:actions>
      </.simple_form>
    </div>
    """
  end

  @impl true
  def update(%{link: link} = assigns, socket) do
    {:ok,
     socket
     |> assign(assigns)
     |> assign_new(:form, fn ->
       to_form(Links.edit_one(link))
     end)}
  end

  @impl true
  def handle_event("validate", %{"link" => link_params}, socket) do
    link_params = with_user_id(link_params, socket.assigns.current_user)
    changeset = Links.edit_one(socket.assigns.link, link_params)
    {:noreply, assign(socket, form: to_form(changeset, action: :validate))}
  end

  def handle_event("save", %{"link" => link_params}, socket) do
    save_link(socket, socket.assigns.action, link_params)
  end

  defp save_link(socket, :edit, link_params) do
    case Links.update_one(socket.assigns.link, link_params) do
      {:ok, link} ->
        notify_parent({:saved, link})

        {:noreply,
         socket
         |> put_flash(:info, "Link updated successfully")
         |> push_patch(to: socket.assigns.patch)}

      {:error, %Ecto.Changeset{} = changeset} ->
        {:noreply, assign(socket, form: to_form(changeset))}
    end
  end

  defp save_link(socket, :new, link_params) do
    link_params =
      link_params
      |> with_user_id(socket.assigns.current_user)

    case Links.insert_one(link_params) do
      {:ok, link} ->
        notify_parent({:saved, link})

        {:noreply,
         socket
         |> put_flash(:info, "Link created successfully")
         |> push_patch(to: socket.assigns.patch)}

      {:error, %Ecto.Changeset{} = changeset} ->
        {:noreply, assign(socket, form: to_form(changeset))}
    end
  end

  defp notify_parent(msg), do: send(self(), {__MODULE__, msg})

  defp with_user_id(params, current_user) do
    if Map.has_key?(params, "user_id") do
      params
    else
      params
      |> Map.put("user_id", current_user.id)
    end
  end
end
